@@ -1,16 +1,38 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/*
+ * Copyright (C) 2001, 2002 Anders Carlsson <andersca@gnu.org>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
+#include <config.h>
+
 #include <string.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include "gconf-tree-model.h"
 
 typedef struct _Node Node;
 
 #define NODE(x) ((Node *)(x))
 
-static void gconf_tree_model_clear_node (GtkTreeModel *tree_model, Node *node);
-
-static GdkPixbuf *closed_icon;
-static GdkPixbuf *open_icon;
+static void gconf_tree_model_class_init       (GConfTreeModelClass *klass);
+static void gconf_tree_model_init             (GConfTreeModel *model);
+static void gconf_tree_model_tree_model_init  (GtkTreeModelIface *iface);
+static void gconf_tree_model_clear_node       (GtkTreeModel *tree_model, Node *node);
 
+G_DEFINE_TYPE_WITH_CODE (GConfTreeModel, gconf_tree_model, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL, gconf_tree_model_tree_model_init));
 
 struct _Node {
 	gint ref_count;
@@ -23,6 +45,7 @@ struct _Node {
 	Node *prev;
 };
 
+
 GtkTreePath *
 gconf_tree_model_get_tree_path_from_gconf_path (GConfTreeModel *tree_model, const char *key)
 {
@@ -41,7 +64,7 @@ gconf_tree_model_get_tree_path_from_gconf_path (GConfTreeModel *tree_model, cons
 
 	key_array = g_strsplit (key + 1, "/", 0);
 
-	if (!gtk_tree_model_get_iter_root (GTK_TREE_MODEL (tree_model), &iter)) {
+	if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (tree_model), &iter)) {
 
 		/* Ugh, something is terribly wrong */
 		g_strfreev (key_array);
@@ -171,9 +194,6 @@ gconf_tree_model_get_column_type (GtkTreeModel *tree_model, gint index)
 	switch (index) {
 	case GCONF_TREE_MODEL_NAME_COLUMN:
 		return G_TYPE_STRING;
-	case GCONF_TREE_MODEL_CLOSED_ICON_COLUMN:
-	case GCONF_TREE_MODEL_OPEN_ICON_COLUMN:
-		return GDK_TYPE_PIXBUF;
 	default:
 		return G_TYPE_INVALID;
 	}
@@ -267,15 +287,6 @@ gconf_tree_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint co
 			g_value_set_string (value, ptr);
 		}
 		break;
-	case GCONF_TREE_MODEL_CLOSED_ICON_COLUMN:
-		g_value_init (value, GDK_TYPE_PIXBUF);
-		g_value_set_object (value, closed_icon);
-		break;
-
-	case GCONF_TREE_MODEL_OPEN_ICON_COLUMN:
-		g_value_init (value, GDK_TYPE_PIXBUF);
-		g_value_set_object (value, open_icon);
-		break;
 	default:
 		break;
 	}
@@ -410,9 +421,12 @@ gconf_tree_model_iter_has_child (GtkTreeModel *tree_model, GtkTreeIter *iter)
 
 	if (list == NULL)
 		return FALSE;
-	else
-		return TRUE;
+	else {
+		g_slist_foreach (list, (GFunc) g_free, NULL);
+		g_slist_free (list);
 
+		return TRUE;
+	}
 }
 
 static void
@@ -474,11 +488,23 @@ gconf_tree_model_tree_model_init (GtkTreeModelIface *iface)
 	iface->unref_node = gconf_tree_model_unref_node;
 }
 
+static void
+gconf_tree_model_finalize (GObject *obj)
+{
+	GConfTreeModel *model = GCONF_TREE_MODEL (obj);
+
+	if (model->client) {
+		g_object_unref (model->client);
+		model->client = NULL;
+	}
+
+	G_OBJECT_CLASS (gconf_tree_model_parent_class)->finalize (obj);
+}
+
 static void
 gconf_tree_model_class_init (GConfTreeModelClass *klass)
 {
-	closed_icon = gdk_pixbuf_new_from_file (GCONF_EDITOR_IMAGEDIR"/folder-closed.png", NULL);
-	open_icon = gdk_pixbuf_new_from_file (GCONF_EDITOR_IMAGEDIR"/folder-open.png", NULL);
+	G_OBJECT_CLASS (klass)->finalize = gconf_tree_model_finalize;
 }
 
 static void
@@ -501,46 +527,8 @@ gconf_tree_model_set_client (GConfTreeModel *model, GConfClient *client)
 	model->client = client;
 }
 
-GType
-gconf_tree_model_get_type (void)
-{
-	static GType object_type = 0;
-
-	if (!object_type) {
-		static const GTypeInfo object_info = {
-			sizeof (GConfTreeModelClass),
-			NULL,		/* base_init */
-			NULL,		/* base_finalize */
-			(GClassInitFunc) gconf_tree_model_class_init,
-			NULL,		/* class_finalize */
-			NULL,		/* class_data */
-			sizeof (GConfTreeModel),
-			0,              /* n_preallocs */
-			(GInstanceInitFunc) gconf_tree_model_init
-		};
-
-		static const GInterfaceInfo tree_model_info = {
-			(GInterfaceInitFunc) gconf_tree_model_tree_model_init,
-			NULL,
-			NULL
-		};
-
-		object_type = g_type_register_static (G_TYPE_OBJECT, "GConfTreeModel", &object_info, 0);
-
-		g_type_add_interface_static (object_type,
-					     GTK_TYPE_TREE_MODEL,
-					     &tree_model_info);
-	}
-
-	return object_type;
-}
-
 GtkTreeModel *
 gconf_tree_model_new (void)
 {
-	GConfTreeModel *model;
-
-	model = g_object_new (GCONF_TYPE_TREE_MODEL, NULL);
-
-	return GTK_TREE_MODEL (model);
+	return GTK_TREE_MODEL (g_object_new (GCONF_TYPE_TREE_MODEL, NULL));
 }
